from time import ctime
import os

from Framework import Controller
from Framework import templatepage
from Framework.PluginManager import DeclarePlugin

from cherrypy import expose

from runDataProviders import *
from jobDataProviders import *
from fileDataProviders import *
from guiProvider import *
from utils import *
from consts import *

import globals
import config
import sqlManager

# view ids

RUNS_T = 'runsT'
REPACKER_T = 'repackerT'
REPACK_S_T = 'repackST'
MERGE_T = 'mergeT'
MERGE_S_T = 'mergeST'
RECO_T = 'recoT'
RECO_S_T = 'recoST'
RUN_REPACK_T = 'runRepackT'
RUN_REPACK_MERGE_T = 'runRepackMergeT'
RUN_RECO_T = 'runRecoT'
RUN_STREAMER_T = 'runStreamerT'
RUN_REPACK_STATS_T = 'runRepackStatsT'
RUN_MERGE_STATS_T = 'runMergeStatsT'
RUN_RECO_STATS_T = 'runRecoStatsT'
JOBS_T = 'jobsT'

TMPL_PATH = os.path.join(__file__.rsplit ("/", 1)[0], "Templates/")

class T0Mon (Controller):
    #### Constructors ####

    def __init__(self, context):
        Controller.__init__(self, context, __file__)
        globals.logger = self.context.Logger()
        sqlManager.initBasicConnection()

    #### Public ####

    @expose

    def testConfig(self):
        status = "# Starting config file check<br />"
        if config.DB_PASSWORD == None or config.DB_PASSWORD == "":
            status += "Warning: Your database password is empty<br />"
        status += "# Finished config file check<br/>"
        return status

    @expose
    def streamer(self, id, **kwds):
        globals.kwds = kwds
        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad streamer id",
                    "tmplPath" : TMPL_PATH
                }
            )

        streamerData = StreamerByIdsDP([id])
        streamerFiles = getTableGui('streamerFiles', streamerData)
        streamerFiles = '<div style="border:2px solid orange">' + streamerFiles + "</div>"

#        streamerIds = [int(line[0]) for line in streamerData.data]
#        repackData = RepackJobByStreamerDP(streamerIds)
#        repackJobs = getTableGui('repackJobs', repackData)

#        jobIds = [int(line[0]) for line in repackData.data]
#        repackData = RepackByRepackJobIdsDP(jobIds)
#        repackFiles = getTableGui('repackFiles', repackData)

#        repackIds = [int(line[0]) for line in repackData.data]
#        mergeData = MergeJobByRepackIds(repackIds)
#        mergeJobs = getTableGui('mergeJobs', mergeData)

##        jobIds = [int(line[0]) for line in mergeData.data]
#        mergeData = RepackMergeByRepackIdsDP(repackIds)
#        mergeFiles = getTableGui('mergeFiles', mergeData)

        return self.templatePage (
                "history", {
                    'streamerFiles': streamerFiles,
#                    'repackJobs' : repackJobs,
#                    'repackFiles' : repackFiles,
#                    'mergeJobs' : mergeJobs,
#                    'mergeFiles' : mergeFiles,
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def jobs(self, **kwds):
        """
        Page displaying job lists
        """
        globals.kwds = kwds
        error = ""
        table = ""
        if globals.kwds.has_key('table'):
            table = str(kwds['table'])
        else: error += "<br />No table name specified"
        if globals.kwds.has_key('status'):
            try:
                status = int(kwds['status'])
            except ValueError:
                error += "<br />Status is not an int"
        else: error += "<br />No status flag specified"
        if globals.kwds.has_key('runid'):
            try:
                runid = int(kwds['runid'])
            except ValueError:
                error += "<br />Run id is not an int"
        else: runid = None

        params = self._getTableParams(JOBS_T)

        if table != "" and error == "":
            if table == TABLES.REPACK_JOBS:
                if runid != None:
                    jtd = RepackJobByStatusAndRunIdDP(status, runid, params.orderby, params.asc)
                else:
                    jtd = RepackJobByStatusDP(status, params.orderby, params.asc)
            elif table == TABLES.RECO_JOBS:
                if runid != None:
                    jtd = RecoJobByStatusAndRunIdDP(status, runid, params.orderby, params.asc)
                else:
                    jtd = RecoJobByStatusDP(status, params.orderby, params.asc)
            elif table == TABLES.MERGE_JOBS:
                if runid != None:
                    jtd = RepackMergeJobByStatusAndRunIdDP(status, runid, params.orderby, params.asc)
                else:
                    jtd = RepackMergeJobByStatusDP(status, params.orderby, params.asc)
            else: error += "<br />Bad jobs table name specified"

        if error == "":
            jt = getTableGui(JOBS_T, jtd)
        else:
            jt = error

        return self.templatePage("jobs", {
            JOBS_T: jt,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def run(self, runid, **kwds):
        """
        Page displaying run statistics
        """
        globals.kwds = kwds

        try:
            runid = int(runid)
        except ValueError:
            return self.templatePage ("error", {
                'msg': formatExceptionInfo(),
                "tmplPath" : TMPL_PATH
            })

        params = self._getTableParams(RUN_REPACK_MERGE_T)

        try:
            rrmtd = RepackMergeByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrmt = getTableGui(RUN_REPACK_MERGE_T, rrmtd)
        except: rrmt = formatExceptionInfo()

        params = self._getTableParams(RUN_REPACK_T)

        try:
            rrtd = RepackByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrt = getTableGui(RUN_REPACK_T, rrtd)
        except: rrt = formatExceptionInfo()

        params = self._getTableParams(RUN_RECO_T)

        try:
            rrtd = RunRecoDataProvider(runid, params.orderby, params.asc, params.page)
            rret = getTableGui(RUN_RECO_T, rrtd)
        except: rret = formatExceptionInfo()

        params = self._getTableParams(RUN_STREAMER_T)

        try:
            rstd = StreamerByRunIdDP(runid, params.orderby, params.asc, params.page)
            rst = getTableGui(RUN_STREAMER_T, rstd)
        except: rst = formatExceptionInfo()

        try:
            rrstd = RepackStatsDataProvider(runid)
            rrst = getVertTableGui(RUN_REPACK_T, rrstd)
        except: rrst = formatExceptionInfo()

        try:
            rmstd = MergeStatsDataProvider(runid)
            rmst = getVertTableGui(RUN_REPACK_T, rmstd)
        except: rmst = formatExceptionInfo()

        try:
            rcstd = RecoStatsDataProvider(runid)
            rcst = getVertTableGui(RUN_REPACK_T, rcstd)
        except: rcst = formatExceptionInfo()

        return self.templatePage("run", {
            RUN_REPACK_T : rrt,
            RUN_RECO_T : rret,
            RUN_STREAMER_T : rst,
            RUN_REPACK_STATS_T : rrst,
            RUN_MERGE_STATS_T : rmst,
            RUN_RECO_STATS_T : rcst,
            RUN_REPACK_MERGE_T : rrmt,
            #"streamRepackG" : srg,
            "runid" : runid,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def index(self, **kwds):
        """
        Main page
        """

        globals.kwds = kwds

        #### Runs table ####

        params = self._getTableParams(RUNS_T)

        try:
            runsTableTd = RunsDataProvider(params.orderby, params.asc, params.page)
            runsTable = getTableGui(RUNS_T, runsTableTd, 100)
        except: runsTable = formatExceptionInfo()

        #### Repacking status table ####

        params = self._getTableParams(REPACKER_T)

        try:
            repackerTd = RepackRunningJobDP(params.orderby, params.asc)
            repackerTable = getTableGui(REPACKER_T, repackerTd)
        except: repackerTable = formatExceptionInfo()

        #### Reconstruction status table ####

        params = self._getTableParams(RECO_T)

        try:
            recoTd = RecoRunningJobDP(params.orderby, params.asc)
            recoTable = getTableGui(RECO_T, recoTd)
        except: recoTable = formatExceptionInfo()

        #### Merging status table ####

        params = self._getTableParams(MERGE_T)

        try:
            mergeTd = RepackMergeRunningJobDP(params.orderby, params.asc)
            mergeTable = getTableGui(MERGE_T, mergeTd)
        except: mergeTable = formatExceptionInfo()

        try:
            repackStatsTd = StatsDataProvider("Repack stats", TABLES.REPACK_JOBS)
            repackStatsTd.load()
            repackStatsTable = getVertTableGui(REPACK_S_T, repackStatsTd)
        except: repackStatsTable = formatExceptionInfo()

        try:
            recoStatsTd = StatsDataProvider("Reco stats", TABLES.RECO_JOBS)
            recoStatsTd.load()
            recoStatsTable = getVertTableGui(RECO_S_T, recoStatsTd)
        except: recoStatsTable = formatExceptionInfo()

        try:
            mergeStatsTd = StatsDataProvider("Merge stats", TABLES.MERGE_JOBS)
            mergeStatsTd.load()
            mergeStatsTable = getVertTableGui(MERGE_S_T, mergeStatsTd)
        except: mergeStatsTable = formatExceptionInfo()

        return self.templatePage("index", {
            RUNS_T : runsTable,
            REPACKER_T : repackerTable,
            RECO_T : recoTable,
            MERGE_T : mergeTable,
            MERGE_S_T : mergeStatsTable,
            REPACK_S_T : repackStatsTable,
            RECO_S_T : recoStatsTable,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    #### Private ####

    def _getTableParams(self, tableid, orderby = 0, asc = False, page = 0):
        """
        Method for parsing table params from GET
        """
        if globals.kwds.has_key(tableid + "_p"):
            try:
                pages = globals.kwds[tableid + "_p"]
                if pages == "all": page = None
                else:
                    page = int(globals.kwds[tableid + "_p"])
                    if page < 0:
                        page = 0
                        self.context.Logger().error("Page num < 0")
            except ValueError:
                self.context.Logger().error("Could not convert page num to an int")

        if globals.kwds.has_key(tableid):
            words = globals.kwds[tableid].rsplit("_")
            try:
                orderby = int(words[0])
                if orderby < 0:
                    orderby = 0
                    self.context.Logger().error("Col id < 0")

            except ValueError:
                self.context.Logger().error("Could not convert table row id to an int")

            if (len(words) == 2):
                if (words[1] == "True"):
                    asc = True
                else: asc = False
            else:
                self.context.Logger().error("Badly formed table params")

        params = TableParams(orderby, asc, page)

        return params

class TableParams:
    def __init__(self, orderby = 0, asc = False, page = 0):
        self.orderby = orderby
        self.asc = asc
        self.page = page

DeclarePlugin ("/Controllers/" + PROJECT_NAME + "/" + PROJECT_NAME, T0Mon, {"baseUrl": "/" + PROJECT_NAME})

