from sqlalchemy import *

from utils import *
from analysis import *
from consts import *
from sqlManager import *

import globals
import config

class Cols:
    def __init__(self, cols):
        self.cols = cols
        self.colsMap = {}
        for col in self.cols:
            self.colsMap[col.id] = col

    def __len__(self):
        return len(self.cols)

    def getId(self, id):
        if type(id) == int:
            return self.cols[id].id
        else:
            for i in range(len(self.cols)):
                if self.cols[i].id == id:
                    return i
            raise Exception("Column with id " + str(id) + " not found")

    def __setitem__(self, id, col):
        if type(id) == int:
            self.cols[id] = col
        else:
            for i in range(len(self.cols)):
                if self.cols[i].id == id:
                    self.cols[i] = col
                    break
            raise Exception("Column with id " + str(id) + " not found")

    def __getitem__(self, id):
        if type(id) == int:
            return self.cols[id]
        else:
            return self.colsMap[id]

class Col:
    def __init__(self, id, title = None, decorator = None, info = None):
        self.id = id
        if title == None:
            self.title = self.id
        else:
            self.title = title
        self.decorator = decorator
        self.info = info

class TableDataProvider:
    """
    Parent class for all table data providers
    """
    def __init__(self, orderby = 0, asc = True):
        self.orderby = orderby
        self.asc = asc
        self.bindParams = None

    """
    Template method, childs have to overload methods that they need
    """
    def _load(self):
        self._buildQuery()
        self._validate()
        self._execute()
        start = datetime.datetime.now()
        self._analysis()
        delta = datetime.datetime.now() - start
        self.atime = delta.seconds * 1000 + delta.microseconds / float(1000)

    def _analysis(self):
        return None

    def _execute(self):
        sqlResult = executeQuery(self.query, self.bindParams)
        self.data = self._sqlResultToData(sqlResult)
        self.qtime = sqlResult.time
        sqlResult.close()

    def _validate(self):
        if (self.orderby < 0 or self.orderby >= len(self.cols)):
            globals.logger.log("Bad orderby col number provided for " + str(self))
            self.orderby = 0

    def _sqlResultToData(self, sqlResult):
        data = []
        for line in sqlResult: data.append(list(line))
        return data

class PagedTableDataProvider(TableDataProvider):

    def __init__(self, orderby, asc, page = None, pageSize = config.PAGE_SIZE, dataCount = None):
        TableDataProvider.__init__(self, orderby, asc)
        self.page = page
        self.pageSize = pageSize
        self.dataCount = dataCount

    def _buildQuery(self):
        self._buildMainQuery()
        if self.dataCount == None:
            self._buildCountQuery()

    def _validate(self):
        TableDataProvider._validate(self)
        if self.page != None and self.page < 0:
            globals.logger.log("Bad page number provided for " + str(self))
            self.page = None

    def _execute(self):
        if self.page != None:
            if type(self.mainQuery) == str:
                self.mainQuery = """SELECT * FROM (""" + \
                    """ SELECT
                        a.*,
                        rownum rnum
                        FROM
                        (""" + str(self.mainQuery) + """) a
                        WHERE rownum <= """ + str(int(self.page * self.pageSize + self.pageSize)) + \
                        """)
                        WHERE rnum > """ + str(int(self.page * self.pageSize))
            else:
#                globals.logger.log("Processing main query")
                self.mainQuery = self.mainQuery.apply_labels()
#                self.mainQuery = select([func.all()], from_obj=[self.mainQuery])
                self.mainQuery = self.mainQuery. \
                    offset(self.page * self.pageSize). \
                    limit(self.pageSize)
#                globals.logger.log(self.mainQuery._offset)
#        globals.logger.log(self.mainQuery)
        sqlResult = executeQuery(self.mainQuery, self.bindParams)
        self.data = self._sqlResultToData(sqlResult)
        self.qtime = sqlResult.time
        sqlResult.close()
        if self.dataCount == None:
            sqlResult = executeQuery(self.countQuery)
            self.dataCount = int(sqlResult.fetchone()[0])
            self.qtime += sqlResult.time
            sqlResult.close()
