from sqlalchemy import *

from dataProviderBase import *
from utils import *
from analysis import *
from consts import *
from sqlManager import *

import globals
import config

class RepackDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
        Col("Status"),
        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)

    def _buildCountQuery(self):
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        parentageTable = Table(TABLES.REPACKED_MERGE_PARENTAGE, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        self.countQuery = \
            select(
                [func.count(repackedTable.c.repacked_id)],
                self.whereCond
            )

    def _buildMainQuery(self):
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        rStatusTable = Table(TABLES.REPACKED_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        eStatusTable = Table(TABLES.EXPORT_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        datasetTable = Table(TABLES.PRIMARY_DATASET, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        parentageTable = Table(TABLES.REPACKED_MERGE_PARENTAGE, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        c = (
            repackedTable.c.repacked_id,
            repackedTable.c.filesize,
            repackedTable.c.events,
            rStatusTable.c.status,
            eStatusTable.c.status,
            datasetTable.c.name,
            repackedTable.c.lfn
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
            from_obj = [repackedTable.
                            outerjoin(rStatusTable, rStatusTable.c.id == repackedTable.c.status).
                            outerjoin(eStatusTable, repackedTable.c.export_status == eStatusTable.c.id).
                            outerjoin(datasetTable, datasetTable.c.dataset_id == repackedTable.c.dataset_id)
                        ]
            ).order_by(o)

class RepackByRunIdDP(RepackDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        RepackDP.__init__(self, orderby, asc, page)

        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        parentageTable = Table(TABLES.REPACKED_MERGE_PARENTAGE, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        self.whereCond = and_(
            repackedTable.c.run_id == runid,
            not_(repackedTable.c.repacked_id.in_(select([parentageTable.c.output_id])))
        )
        RepackDP._load(self)
        self.title = str(self.dataCount) + " repacked files"

class RepackByRepackJobIdsDP(RepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        RepackDP.__init__(self, orderby, asc, page)

        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        assocTable = Table(TABLES.REPACK_STREAMER_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        jobAssocTable = Table(TABLES.JOB_DATASET_STREAMER_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        parentageTable = Table(TABLES.REPACKED_MERGE_PARENTAGE, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        self.whereCond = and_(
            repackedTable.c.repacked_id.in_(
                select ([assocTable.c.repacked_id],
                    assocTable.c.streamer_id.in_(
                        select([jobAssocTable.c.streamer_id],
                            jobAssocTable.c.job_id.in_(ids))
                    )
                )
            ),
            not_(repackedTable.c.repacked_id.in_(select([parentageTable.c.output_id])))
        )
        RepackDP._load(self)
#        globals.logger.log(self.mainQuery)
        self.title = "Repacked files"

class RepackMergeDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
        Col("Status"),
        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)

    def _buildCountQuery(self):
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        self.countQuery = select([func.count('*')],
            self.whereCond
        )

    def _buildMainQuery(self):
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        rStatusTable = Table(TABLES.REPACKED_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        eStatusTable = Table(TABLES.EXPORT_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        datasetTable = Table(TABLES.PRIMARY_DATASET, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        c = (
            repackedTable.c.repacked_id,
            repackedTable.c.filesize,
            repackedTable.c.events,
            rStatusTable.c.status,
            eStatusTable.c.status,
            datasetTable.c.name,
            repackedTable.c.lfn
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
            from_obj = [repackedTable.
                            outerjoin(rStatusTable, rStatusTable.c.id == repackedTable.c.status).
                            outerjoin(eStatusTable, repackedTable.c.export_status == eStatusTable.c.id).
                            outerjoin(datasetTable, datasetTable.c.dataset_id == repackedTable.c.dataset_id)
                        ]
            ).order_by(o)

class RepackMergeByRepackIdsDP(RepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        RepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        assocTable = Table(TABLES.MERGE_JOB_REPACK_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        parentageTable = Table(TABLES.REPACKED_MERGE_PARENTAGE, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = \
            and_(
                or_(
                    repackedTable.c.status == REPACK_RECONSTRUCTION_QUEUED,
                    repackedTable.c.status == REPACK_RECONSTRUCTION_DISABLED
                ),
                or_(
                    repackedTable.c.repacked_id.in_(
                        select([parentageTable.c.output_id],
                            parentageTable.c.input_id.in_(ids)
                        )
                    ),
                    repackedTable.c.repacked_id.in_(ids)
                )
            )
        RepackMergeDP._load(self)
        self.title = "Merge files"

class RepackMergeByRunIdDP(RepackMergeDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        RepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = \
            and_(
                or_(
                    repackedTable.c.status == REPACK_RECONSTRUCTION_QUEUED,
                    repackedTable.c.status == REPACK_RECONSTRUCTION_DISABLED
                ),
                repackedTable.c.run_id == runid
            )
        RepackMergeDP._load(self)
        self.title = str(self.dataCount) + " merge files"

class RunRecoDataProvider(PagedTableDataProvider):

    cols = Cols([
        Col("Id"),
        Col("Filesize", decorator=humanFilesize),
        Col("Events"),
        Col("Status"),
        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.runid = runid
        PagedTableDataProvider._load(self)
        self.title = str(self.dataCount) + " reconstructed files"\

    def _buildCountQuery(self):
        recoTable = Table(TABLES.RECONSTRUCTED, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        self.countQuery = select(
            [func.count(recoTable.c.reconstructed_id)],
            recoTable.c.run_id == self.runid
        )

    def _buildMainQuery(self):
        recoTable = Table(TABLES.RECONSTRUCTED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        rStatusTable = Table(TABLES.RECONSTRUCTED_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        eStatusTable = Table(TABLES.EXPORT_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        datasetTable = Table(TABLES.PRIMARY_DATASET, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        c = (
            recoTable.c.reconstructed_id,
            recoTable.c.filesize,
            recoTable.c.events,
            rStatusTable.c.status,
            eStatusTable.c.status,
            datasetTable.c.name,
            recoTable.c.lfn
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            recoTable.c.run_id == self.runid,
            from_obj = [
                recoTable.
                outerjoin(rStatusTable, rStatusTable.c.id == recoTable.c.status).
                outerjoin(eStatusTable, recoTable.c.export_status == eStatusTable.c.id).
                outerjoin(datasetTable, datasetTable.c.dataset_id == recoTable.c.dataset_id)
            ]
            ).order_by(o)

class StreamerDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id"),
        Col("Lumi Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
        Col("Name"),
        Col("E"),
        Col("S"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)

    def _buildCountQuery(self):
        streamerTable = Table(TABLES.STREAMER, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        self.countQuery = select(
            [func.count(streamerTable.c.streamer_id)],
            self.whereCond
        )

    def _buildMainQuery(self):
        streamerTable = Table(TABLES.STREAMER, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        c = (
            streamerTable.c.streamer_id,
            streamerTable.c.lumi_id,
            streamerTable.c.filesize,
            streamerTable.c.events,
            streamerTable.c.streamname,
            streamerTable.c.exportable,
            streamerTable.c.splitable,
            streamerTable.c.lfn
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond
        ).order_by(o)

class StreamerByRunIdDP(StreamerDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Table(TABLES.STREAMER, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = (streamerTable.c.run_id == runid)
        StreamerDP._load(self)
        self.title = str(self.dataCount) + " streamer files"

class StreamerByIdsDP(StreamerDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Table(TABLES.STREAMER, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = (streamerTable.c.streamer_id.in_(ids))
        StreamerDP._load(self)
        self.title = "Streamer files"

