from sqlalchemy import *

from dataProviderBase import *
from utils import *
from analysis import *
from consts import *
from sqlManager import *

import globals
import config

class StatsDataProvider:

    def __init__(self, title, tableName):
        self.title = title
        self.tableName = tableName
        self.qtime = 0;
        self.data = []
        self.data.append([])
        self.bindParams = None

    def _buildCols(self):
        self.cols = _getStatsCols(self.tableName)

    def _buildQuery(self, status):
        jobTable = Table(self.tableName, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.query = select([func.count(jobTable.c.job_status)], jobTable.c.job_status == status)

    def load(self):
        self._buildCols()
        for i in range (JOB_NEW, JOB_ABANDONED + 1):
            self._buildQuery(i)
            sqlResult = executeQuery(self.query, self.bindParams)
            self.data[0].append(sqlResult.fetchone()[0])
            self.qtime += sqlResult.time
            sqlResult.close()

class RepackStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "Repack stats"):
        StatsDataProvider.__init__(self, title, TABLES.REPACK_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = _getStatsCols(self.tableName, self.runid)

    def _buildQuery(self, status):
#        jobTable = Table(self.tableName, globals.meta, autoload = True, schema = config.DB_SCHEMA)
#        assocTable = Table(TABLES.JOB_DATASET_STREAMER_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
#        streamerTable = Table(TABLES.STREAMER, globals.meta, autoload = True, schema = config.DB_SCHEMA)
#        self.query = select(
#            [func.count(distinct(jobTable.c.job_id))],
#            and_(
#                jobTable.c.job_status == status,
#                streamerTable.c.run_id == self.runid
#            ),
#            from_obj = [
#                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
#                    outerjoin(streamerTable, assocTable.c.streamer_id == streamerTable.c.streamer_id)
#            ]
#        )
        schema = ""
        if config.DB_SCHEMA != "" and config.DB_SCHEMA != None:
            schema = config.DB_SCHEMA + "."
        jobTable = schema + self.tableName
        assocTable = schema + TABLES.JOB_DATASET_STREAMER_ASSOC
        streamerTable = schema + TABLES.STREAMER
        repackedTable = schema + TABLES.REPACKED

        self.query = """
            SELECT COUNT(DISTINCT(j.job_id)) FROM
            """ + jobTable + """ j
            LEFT OUTER JOIN """ + assocTable + """ a ON a.job_id = j.job_id
            LEFT OUTER JOIN """ + streamerTable + """ s ON s.streamer_id = a.streamer_id
            WHERE j.job_status = :status AND s.run_id = :runid
        """

        self.bindParams = {'runid' : int(self.runid), 'status' : int(status)}


class MergeStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "Merge stats"):
        StatsDataProvider.__init__(self, title, TABLES.MERGE_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = _getStatsCols(self.tableName, self.runid)

    def _buildQuery(self, status):
        jobTable = Table(TABLES.MERGE_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        assocTable = Table(TABLES.MERGE_JOB_REPACK_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.query = select(
            [func.count(distinct(jobTable.c.job_id))],
            and_(
                jobTable.c.job_status == status,
                repackedTable.c.run_id == self.runid
            ),
            from_obj = [
                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                    outerjoin(repackedTable, assocTable.c.repacked_id == repackedTable.c.repacked_id)
            ]
        )
        # globals.logger.log(self.query)

class RecoStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "Reco stats"):
        StatsDataProvider.__init__(self, title, TABLES.RECO_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = _getStatsCols(self.tableName, self.runid)

    def _buildQuery(self, status):
        jobTable = Table(TABLES.RECO_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        assocTable = Table(TABLES.RECO_JOB_REPACK_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.query = select(
            [func.count(distinct(jobTable.c.job_id))],
            and_(
                jobTable.c.job_status == status,
                repackedTable.c.run_id == self.runid
            ),
            from_obj = [
                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                    outerjoin(repackedTable, assocTable.c.repacked_id == repackedTable.c.repacked_id)
            ]
        )
        # globals.logger.log(self.query)

class RepackMergeJobDP(TableDataProvider):

    cols = Cols([
        Col("Job ID"),
        Col("Run ID", decorator=formRunLink),
        Col("Definition time", formatTime),
        Col("Repacked"),
        Col("Dataset")
    ])

    def __init__(self, orderby = 0, asc = True, status = None, runid = None):
        TableDataProvider.__init__(self, orderby, asc)

    def _buildQuery(self):
        jobTable = Table(TABLES.MERGE_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        statusTable = Table(TABLES.JOB_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        assocTable = Table(TABLES.MERGE_JOB_REPACK_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        repackTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        datasetTable = Table(TABLES.PRIMARY_DATASET, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        c = (
            jobTable.c.job_id,
            repackTable.c.run_id,
            jobTable.c.definition_time,
            func.count(assocTable.c.repacked_id),
            datasetTable.c.name,
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.query = select(
            c,
            self.whereCond,
            from_obj = [
                jobTable.
                #outerjoin(statusTable).
                outerjoin(assocTable, jobTable.c.job_id == assocTable.c.job_id).
                outerjoin(repackTable, repackTable.c.repacked_id == assocTable.c.repacked_id).
                outerjoin(datasetTable, datasetTable.c.dataset_id == repackTable.c.dataset_id)
            ]
            ).group_by(
                jobTable.c.job_id,
                jobTable.c.definition_time,
                datasetTable.c.name,
                repackTable.c.run_id,
            ).order_by(o)


class RecoJobDP(TableDataProvider):

    cols = Cols([
        Col("Job ID"),
        Col("Run ID", decorator=formRunLink),
        Col("Definition time", decorator=formatTime),
        Col("Repacked"),
        Col("Dataset"),
    ])

    def __init__(self, orderby = 0, asc = True, status = None, runid = None):
        TableDataProvider.__init__(self, orderby, asc)

    def _buildQuery(self):
        jobTable = Table(TABLES.RECO_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        statusTable = Table(TABLES.JOB_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        assocTable = Table(TABLES.RECO_JOB_REPACK_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        repackTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        datasetTable = Table(TABLES.PRIMARY_DATASET, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        c = (
             jobTable.c.job_id,
             repackTable.c.run_id,
             jobTable.c.definition_time,
             func.count(assocTable.c.repacked_id),
             datasetTable.c.name,
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.query = select(
            c,
            self.whereCond,
            from_obj = [
                jobTable.
                #outerjoin(statusTable).
                outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                outerjoin(repackTable, repackTable.c.repacked_id == assocTable.c.repacked_id).
                outerjoin(datasetTable, datasetTable.c.dataset_id == repackTable.c.dataset_id)
            ]
            ).group_by(
                jobTable.c.job_id,
                jobTable.c.definition_time,
                datasetTable.c.name,
                repackTable.c.run_id,
            ).order_by(o)

class RepackJobDP(TableDataProvider):

    cols = Cols([
        Col("Job ID"),
        Col("Run ID", decorator=formRunLink),
        Col("Definition time", decorator=formatTime),
        Col("Streamers"),
        Col("Stream"),
        # ["Dataset", None]
    ])

    def __init__(self, orderby = 0, asc = True, status = None, runid = None):
        TableDataProvider.__init__(self, orderby, asc)

    def _buildQuery(self):
        jobTable = Table(TABLES.REPACK_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        assocTable = Table(TABLES.JOB_DATASET_STREAMER_ASSOC, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        statusTable = Table(TABLES.JOB_STATUS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        datasetTable = Table(TABLES.PRIMARY_DATASET, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        streamerTable = Table(TABLES.STREAMER, globals.meta, autoload = True, schema = config.DB_SCHEMA)

        c = (
             jobTable.c.job_id,
             streamerTable.c.run_id,
             jobTable.c.definition_time,
             func.count(func.distinct(assocTable.c.streamer_id)),
             streamerTable.c.streamname,
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.query = select(
            c,
            self.whereCond,
            from_obj = [
                jobTable.
                outerjoin(assocTable, jobTable.c.job_id == assocTable.c.job_id).
                outerjoin(streamerTable, streamerTable.c.streamer_id == assocTable.c.streamer_id)
            ]
            ).group_by(
                jobTable.c.job_id,
                jobTable.c.definition_time,
                streamerTable.c.run_id,
                streamerTable.c.streamname,
            ).order_by(o)

class RepackRunningJobDP(RepackJobDP):

    cols = Cols([
        Col("Job ID"),
        Col("Run ID", decorator=formRunLink),
        Col("Running time", decorator=deltaTime),
        Col("Streamers"),
        Col("Stream"),
        # ["Dataset", None]
    ])

    def __init__(self, orderby = 0, asc = True):
        RepackJobDP.__init__(self, orderby, asc)
        jobTable = Table(TABLES.REPACK_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = or_(
            jobTable.c.job_status == JOB_USED,
            jobTable.c.job_status == JOB_NEW,
        )
        RepackJobDP._load(self)
        self.title = str(len(self.data)) + " repack jobs running"

class RepackMergeRunningJobDP(RepackMergeJobDP):

    cols = Cols([
        Col("Job ID"),
        Col("Run ID", decorator=formRunLink),
        Col("Running time", decorator=deltaTime),
        Col("Streamers"),
        Col("Stream"),
        # ["Dataset", None]
    ])

    def __init__(self, orderby = 0, asc = True):
        RepackMergeJobDP.__init__(self, orderby, asc)
        jobTable = Table(TABLES.MERGE_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = or_(
            jobTable.c.job_status == JOB_USED,
            jobTable.c.job_status == JOB_NEW,
        )
        RepackMergeJobDP._load(self)
        self.title = str(len(self.data)) + " merge jobs running"

class RecoRunningJobDP(RecoJobDP):

    cols = Cols([
        Col("Job ID"),
        Col("Run ID", decorator=formRunLink),
        Col("Running time", decorator=deltaTime),
        Col("Streamers"),
        Col("Stream"),
    ])

    def __init__(self, orderby = 0, asc = True):
        RecoJobDP.__init__(self, orderby, asc)
        jobTable = Table(TABLES.RECO_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = or_(
            jobTable.c.job_status == JOB_USED,
            jobTable.c.job_status == JOB_NEW,
        )
        RecoJobDP._load(self)
        self.title = str(len(self.data)) + " reco jobs running"

class RepackJobByStatusDP(RepackJobDP):

    def __init__(self, status, orderby = 0, asc = True):
        RepackJobDP.__init__(self, orderby, asc)
        jobTable = Table(TABLES.REPACK_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = jobTable.c.job_status == status
        RepackJobDP._load(self)
        s = JOB_STATUS[status]
        self.title = "Repacker " + s + " jobs"

class RepackMergeJobByStatusDP(RepackMergeJobDP):

    def __init__(self, status, orderby = 0, asc = True):
        RepackMergeJobDP.__init__(self, orderby, asc)
        jobTable = Table(TABLES.MERGE_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = jobTable.c.job_status == status
        RepackMergeJobDP._load(self)
        s = JOB_STATUS[status]
        self.title = "Merge " + s + " jobs"

class RecoJobByStatusDP(RecoJobDP):

    def __init__(self, status, orderby = 0, asc = True):
        RecoJobDP.__init__(self, orderby, asc)
        jobTable = Table(TABLES.RECO_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = jobTable.c.job_status == status
        RecoJobDP._load(self)
        s = JOB_STATUS[status]
        self.title = "Reco " + s + " jobs"

class RepackJobByStatusAndRunIdDP(RepackJobDP):

    def __init__(self, status, runid, orderby = 0, asc = True):
        RepackJobDP.__init__(self, orderby, asc)
        streamerTable = Table(TABLES.STREAMER, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        jobTable = Table(TABLES.REPACK_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = and_(
            jobTable.c.job_status == status,
            streamerTable.c.run_id == runid,
        )
        RepackJobDP._load(self)
        s = JOB_STATUS[status]
        self.title = "Repacker " + s + " jobs for run " + str(runid)

class RepackMergeJobByStatusAndRunIdDP(RepackMergeJobDP):

    def __init__(self, status, runid, orderby = 0, asc = True):
        RepackMergeJobDP.__init__(self, orderby, asc)
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        jobTable = Table(TABLES.MERGE_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = and_(
            jobTable.c.job_status == status,
            repackedTable.c.run_id == runid,
        )
        RepackMergeJobDP._load(self)
        s = JOB_STATUS[status]
        self.title = "Merger " + s + " jobs for run " + str(runid)

class RecoJobByStatusAndRunIdDP(RecoJobDP):

    def __init__(self, status, runid, orderby = 0, asc = True):
        RecoJobDP.__init__(self, orderby, asc)
        repackedTable = Table(TABLES.REPACKED, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        jobTable = Table(TABLES.RECO_JOBS, globals.meta, autoload = True, schema = config.DB_SCHEMA)
        self.whereCond = and_(
            jobTable.c.job_status == status,
            repackedTable.c.run_id == runid,
        )
        RecoJobDP._load(self)
        s = JOB_STATUS[status]
        self.title = "Reco " + s + " jobs for run " + str(runid)

##### private #####

def _getStatsCols(tableName, runid = None):
    return (
            ["New jobs", lambda x: formJobTableLink(x, tableName, JOB_NEW, runid)],
            ["Used jobs", lambda x: formJobTableLink(x, tableName, JOB_USED, runid)],
            ["Successful jobs", lambda x: formJobTableLink(x, tableName, JOB_SUCCESS, runid)],
            ["Failed jobs", lambda x: formJobTableLink(x, tableName, JOB_FAILURE, runid)],
            ["Abandoned jobs", lambda x: formJobTableLink(x, tableName, JOB_ABANDONED, runid)],
        )
